"use strict";

var _commander = _interopRequireDefault(require("commander"));

var _viswizSdk = _interopRequireDefault(require("viswiz-sdk"));

var _Progress = _interopRequireDefault(require("./Progress"));

var _utils = require("./utils");

var _package = _interopRequireDefault(require("../package.json"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint no-console: 0 */
const DEFAULTS = {
  POLL_SECONDS: 5,
  WAIT_SECONDS: 600
};

async function waitForResult(client, projectID, buildID, maxTime) {
  let timeoutTimer;
  const project = await client.getProject(projectID);
  return Promise.race([new Promise((resolve, reject) => {
    timeoutTimer = setTimeout(() => reject(new Error('Waiting for results timed out!')), maxTime * 1000);
  }), new Promise(resolve => {
    let time = 0;

    function poll() {
      (0, _utils.log)(`Waiting for results to be ready... (${(0, _utils.formatTime)(time)}/${(0, _utils.formatTime)(maxTime)})`);
      client.getBuilds(projectID).then(builds => {
        const build = builds.find(item => item.id === buildID);

        if (!build || !build.diffedAt) {
          return schedule();
        }

        clearTimeout(timeoutTimer);
        resolve(build.diffPercentage <= project.diffThreshold);
      }).catch(schedule);
    }

    function schedule() {
      time += DEFAULTS.POLL_SECONDS;
      setTimeout(poll, DEFAULTS.POLL_SECONDS * 1000);
    }

    poll();
  })]);
}

const commands = {
  async build(program, options) {
    const ci = (0, _utils.getCI)();
    const {
      apiKey,
      project: projectID
    } = program;

    if (!apiKey) {
      return (0, _utils.error)('Error: Missing API key!', program);
    }

    if (!projectID) {
      return (0, _utils.error)('Error: Missing project ID!', program);
    }

    if (!options.imageDir) {
      return (0, _utils.error)('Error: Missing image directory!', options);
    }

    if (!options.branch && !ci.branch) {
      return (0, _utils.error)('Error: Missing branch name!', options);
    }

    if (!options.message && !ci.message) {
      return (0, _utils.error)('Error: Missing commit message!', options);
    }

    if (!options.revision && !ci.commit) {
      return (0, _utils.error)('Error: Missing commit revision!', options);
    }

    const client = new _viswizSdk.default(apiKey);
    let progress;
    (0, _utils.log)('Creating build on VisWiz.io...');
    const buildID = await client.buildFolder({
      branch: options.branch || ci.prBranch || ci.branch,
      name: options.message || ci.message,
      projectID,
      revision: options.revision || ci.commit
    }, options.imageDir, (current, total) => {
      if (!progress) {
        progress = new _Progress.default(total, current);
      } else {
        progress.tick();
      }
    });
    const url = `https://app.viswiz.io/projects/${projectID}/build/${buildID}/results`;
    (0, _utils.log)('Build created successfully!');
    (0, _utils.log)(`Build report will be available at: ${url}`);

    if (options.waitForResult) {
      (0, _utils.log)('');
      const passed = await waitForResult(client, projectID, buildID, typeof options.waitForResult === 'string' ? parseInt(options.waitForResult, 10) : DEFAULTS.WAIT_SECONDS);
      (0, _utils.log)('');

      if (!passed) {
        (0, _utils.error)('Build failed!');
      } else {
        (0, _utils.log)('Build passed!');
      }
    }

    return 'OK';
  }

};

function run(argv) {
  _commander.default.version(`${_package.default.version}-${process.platform}-${process.arch}`);

  _commander.default.option('-k, --api-key [apiKey]', 'The API key of a VisWiz account to use. Defaults to VISWIZ_API_KEY env.', process.env.VISWIZ_API_KEY).option('-p, --project [projectID]', 'The ID of a VisWiz project to use. Defaults to VISWIZ_PROJECT_ID env.', process.env.VISWIZ_PROJECT_ID);

  _commander.default.command('build').description('Creates a new build on VisWiz.io and sends images for regression testing.').option('-i, --image-dir <path>', 'The path to a directory (scanned recursively) with images used for the build.').option('-b, --branch [branch name]', 'The branch name for the build. Auto-detected on popular CIs.').option('-m, --message [commit message]', 'The commit message for the build. Auto-detected on popular CIs.').option('-r, --revision [rev]', 'The revision for the build. Auto-detected on popular CIs.').option('-w, --wait-for-result [timeout]', 'Whether to wait for the result of the build comparison (disabled by default). Waits for a maximum number of seconds (defaults to 600).').action((cmd, options) => commands.build(_commander.default, options || cmd).catch(err => {
    (0, _utils.error)(`Error: ${err.message}`);
  }));

  _commander.default.parse(argv);

  if (!_commander.default.args.length) {
    _commander.default.help();
  }
}

module.exports = {
  DEFAULTS,
  commands,
  run
};